@@ -11,23 +11,7 @@ from typing import Iterable
 from packaging.utils import canonicalize_name
 from pdm.backend.base import Builder
 from pdm.backend.hooks import Context
-from pdm.backend.utils import safe_version, to_filename
-
-
-def normalize_file_permissions(st_mode: int) -> int:
-    """
-    Normalizes the permission bits in the st_mode field from stat to 644/755
-
-    Popular VCSs only track whether a file is executable or not. The exact
-    permissions can vary on systems with different umasks. Normalising
-    to 644 (non executable) or 755 (executable) makes builds more reproducible.
-    """
-    # Set 644 permissions, leaving higher bits of st_mode unchanged
-    new_mode = (st_mode | 0o644) & ~0o133
-    if st_mode & 0o100:
-        new_mode |= 0o111  # Executable: 644 -> 755
-
-    return new_mode
+from pdm.backend.utils import normalize_file_permissions, safe_version, to_filename
 
 
 def clean_tarinfo(tar_info: tarfile.TarInfo) -> tarfile.TarInfo:
@@ -58,14 +42,14 @@ class SdistBuilder(Builder):
     target = "sdist"
 
     def get_files(self, context: Context) -> Iterable[tuple[str, Path]]:
-        yield from super().get_files(context)
+        collected = dict(super().get_files(context))
         local_hook = self.config.build_config.custom_hook
         context.ensure_build_dir()
         context.config.write_to(context.build_dir / "pyproject.toml")
-        yield "pyproject.toml", context.build_dir / "pyproject.toml"
+        collected["pyproject.toml"] = context.build_dir / "pyproject.toml"
 
         additional_files: Iterable[str] = filter(
-            None,
+            lambda f: f is not None and f not in collected,
             (
                 local_hook,
                 self.config.metadata.readme_file,
@@ -75,7 +59,8 @@ class SdistBuilder(Builder):
         root = self.location
         for file in additional_files:
             if root.joinpath(file).exists():
-                yield file, root / file
+                collected[file] = root / file
+        return collected.items()
 
     def build_artifact(
         self, context: Context, files: Iterable[tuple[str, Path]]
